#!/usr/bin/env python

from __future__ import print_function

import argparse
import libvirt
import json
import logging

logger = logging.getLogger(__name__)

LIBVIRT_DEFAULT_URI = 'qemu:///system'

LIBVIRT_DEFAULT_NETWORK = 'testing_datalab'

DEFAULT_SPLIT_HOSTNAME = '_'

DEFAULT_SPLIT_GROUP_POS = 1

DEFAULT_COMMON_GROUP = 'docker_hosts,centos_hosts,virtual_machines'

class LibvirtInventory(object):

    def __init__(self,
                 target_network=LIBVIRT_DEFAULT_NETWORK,
                 uri=LIBVIRT_DEFAULT_URI,
                 group_split=DEFAULT_SPLIT_HOSTNAME,
                 group_pos=DEFAULT_SPLIT_GROUP_POS,
                 additional_group=DEFAULT_COMMON_GROUP):

        self.uri = uri.replace("'","").replace('"','')
        self.open_libvirt_connection(self.uri)
        self.parser = {
                                "split": group_split,
                                "pos": group_pos,
        }
        self.common_groups = additional_group.split(",")
        self.set_target_network(target_network)
        self.inventory = {}
        self.set_all_hosts_and_meta()
        self.add_all_hosts_to_groups()
        self.add_all_common_groups()
        self.close_libvirt_connection()


    def open_libvirt_connection(self, uri):
        self.conn = libvirt.openReadOnly(uri)

        if self.conn == None:
            logger.error('Failed to open connection to {0}'.format(uri))
            exit(1)
        else:
            logger.info("Connected to libvirt on remote host: {0}".format(uri))

    def close_libvirt_connection(self):
        self.conn.close()


    def set_target_network(self, target_network):
        self._networks = {network.name(): network for network in self.conn.listAllNetworks()}
        try:
            self._network = self._networks[target_network]
        except KeyError:
            available_networks = ", ".join(self._networks.keys())
            raise ValueError("Target network ({0}) does not exist. Available networks: {1}".format(target_network, available_networks))


    @property
    def network(self):
        return self._network

    def set_all_hosts_and_meta(self):
        self._hosts = { lease["hostname"]: { "ansible_host": lease["ipaddr"] } for lease in self.network.DHCPLeases() }

        self.add_group("all")

        self.inventory["all"]["hosts"] = self.hosts = list(self._hosts.keys())

        self.inventory["_meta"]= {"hostvars": self._hosts}

    def add_group(self, group):
        """ Add a group to inventory """
        if group not in self.inventory:
            self.inventory[group] = dict()

    def get_group(self, group):
        """ Return a group as dictionary from inventory """
        return self.inventory[group]


    def add_child(self, child, group):
        """ Add a child group (string) to group in inventory"""
        if not "children" in self.get_group(group):
            self.inventory[group]["children"] = [child]
        else:
            self.inventory[group]["children"] = list(set([child] + self.inventory[group]["children"]))


    def add_childgroup(self, child_group, parent_group="all"):
        """ Add a group and mark it as child of another group in inventory """
        self.add_group(child_group)
        self.add_child(child_group, parent_group)


    def add_children(self, children, group):
        """ Add list of children to group in inventory"""
        if not "children" in self.get_group(group):
            self.inventory[group]["children"] = list(children)
        else:
            self.inventory[group]["children"] = list(set(children + self.inventory[group]["children"]))


    def add_groupvars(self, vars_dict, group):
        """ Takes a dictionary as argument and add the keys and values as group variables"""
        if "vars" not in self.get_group(group):
            self.inventory[group]["vars"] = vars_dict
        else:
            self.inventory[group]["vars"].update(vars_dict)


    def add_host(self, hostname, group, create=True):
        """ Add one host (string) to a group.
        If create is True (default) then the group will be automatically created if it does not exist yet.
        Else it will fail on KeyError
        """
        if create:
            self.add_group(group)

        if "hosts" not in self.inventory[group]:
            self.inventory[group]["hosts"] = [hostname]
        else:
            self.inventory[group]["hosts"] = list(set([hostname] + self.inventory[group]["hosts"]))

    def add_hosts(self, hostnames, group, create=True):
        """ Add several hosts (list) to a group.
        If create is True (default) then the group will be automatically created if it does not exist yet.
        Else it will fail on KeyError
        """
        if create:
            self.add_group(group)
        self.inventory[group]["hosts"] = list(set(hostnames + self.inventory[group]["hosts"]))

    def add_all_hosts_to_groups(self):
        for hostname in self.hosts:
            prefix, group, last_ip_part = hostname.split(self.parser["split"])
            self.add_host(hostname, group)

    def add_all_common_groups(self):
        for group in self.common_groups:
            self.add_common_group(group)

    def add_common_group(self, common_group):
        self.add_childgroup(common_group)
        for group in self.inventory:
            if group in ["_meta", "all"] + self.common_groups:
                continue
            self.add_child(group, common_group)

    def json_dump(self, indent=None):
        return json.dumps(self.inventory, indent=indent)

def dump_inventory(libvirt_uri, virt_network, split_hostname, group_pos, common_groups):

    inventory = LibvirtInventory(target_network=virt_network,
                                 uri=libvirt_uri,
                                 group_split=split_hostname,
                                 group_pos=group_pos,
                                 additional_group=common_groups)

    return inventory.json_dump(2)

def main():

    parser = argparse.ArgumentParser()

    parser.add_argument("--libvirt_uri",
                        "-l",
                        default=LIBVIRT_DEFAULT_URI,
                        type=str,
                        required=False,
                        help="""URI that python should use to connect to libvirt daemon.
                        Default to '{0}'.

                        Can use several protocols:
                        - qemu:/// for local socket connections
                        - qemu+tcp:// for raw tcp connections
                        - qemu+tls:// for tcp connections over tls encryption

                        More informations available at https://libvirt.org/remote.html
                        """.format(LIBVIRT_DEFAULT_URI))

    parser.add_argument("--virt_network",
                        "-n",
                        default=LIBVIRT_DEFAULT_NETWORK,
                        type=str,
                        required=False,
                        help="""Virtual network from which we want to determine running guests.
                        This script can only list virtual machines located in same virtual network.
                        Default to '{0}'.
                        """.format(LIBVIRT_DEFAULT_NETWORK))


    parser.add_argument("--split_hostname",
                        default=DEFAULT_SPLIT_HOSTNAME,
                        required=False,
                        help="""Which pattern should be used to split the found hostnames.
                        Default to '{0}'.
                        Hostnames are split in order to retrieve group they belong to.
                        This option must be used with `split_group_pos` option.
                        """.format(DEFAULT_SPLIT_HOSTNAME)
                        )


    parser.add_argument("--split_group_pos",
                        default=DEFAULT_SPLIT_GROUP_POS,
                        required=False,
                        type=int,
                        help="""After splitting hostname, what is the index of the group name.
                        Default to '{0}'.
                        This option must be used with `split_hostname` option.
                        """.format(DEFAULT_SPLIT_GROUP_POS)
                        )


    parser.add_argument("--common_groups",
                        "-g",
                        default=DEFAULT_COMMON_GROUP,
                        required=False,
                        type=str,
                        help="""Comma delimited list of group names to which all guests will belong.
                        Default to '{0}'.
                        This option must be used with `split_hostname` option.
                        """.format(DEFAULT_COMMON_GROUP)
                        )

    args = parser.parse_args()

    json_inv = dump_inventory(args.libvirt_uri,
                             args.virt_network,
                             args.split_hostname,
                             args.split_group_pos,
                             args.common_groups)

    print(json_inv)

if __name__ == "__main__":
    main()
