#!/bin/bash

# Ensure that a libvirt volume exists, optionally uploading an image.
# On success, output a JSON object with a 'changed' item.

if [[ $# -ne 5 ]] && [[ $# -ne 6 ]] && [[ $# -ne 7 ]]; then
    echo "Wrong number of arguments. Usage: $0 <name> <pool> <capacity> <format> <image> [<overwrite>] [<qemu_uri>]"
    exit 1
fi

DEFAULT_FORCE_OVERWRITE="False"
DEFAULT_QEMU_URI="qemu:///system"
DEFAULT_QEMU_GROUP="qemu"

NAME=$1
POOL=$2
CAPACITY=$3
FORMAT=$4
IMAGE=$5
FORCE_OVERWRITE=$6
QEMU_URI=$7
QEMU_GROUP=$8

if [ -z "$FORCE_OVERWRITE" ]; then
  FORCE_OVERWRITE=$DEFAULT_FORCE_OVERWRITE
fi

if [ -z "$QEMU_URI" ]; then
  QEMU_URI=$DEFAULT_QEMU_URI
fi

if [ -z "QEMU_GROUP" ]; then
  QEMU_GROUP=$DEFAULT_QEMU_GROUP
fi

# Check whether a volume with this name exists.
output=$(virsh --connect "$QEMU_URI" vol-info --pool $POOL --vol $NAME 2>&1)
result=$?
if [[ $result -eq 0 ]]; then
    if [[ $FORCE_OVERWRITE == "False" ]]; then
      echo '{"changed": false}'
      exit 0
    else
      output_delete=$(virsh --connect "$QEMU_URI" vol-delete --pool $POOL $NAME > /dev/null)
      result_delete=$?
      if [[ $result_delete != "0" ]]; then
        echo "Unexpected error occured. Check stderr_lines"
        exit $result_delete
      fi
    fi
elif ! echo "$output" | grep 'Storage volume not found' >/dev/null 2>&1; then
    echo "Unexpected error while getting volume info"
    echo "$output"
    exit $result
fi

# Create the volume.
output=$(virsh --connect "$QEMU_URI" vol-create-as --pool $POOL --name $NAME --capacity $CAPACITY --format $FORMAT 2>&1)
result=$?
if [[ $result -ne 0 ]]; then
    echo "Failed to create volume"
    echo "$output"
    exit $result
fi

# Determine the path to the volume file.
output=$(virsh --connect "$QEMU_URI" vol-key --pool $POOL --vol $NAME 2>&1)
result=$?
if [[ $result -ne 0 ]]; then
    echo "Failed to get volume file path"
    echo "$output"
    virsh --connect "$QEMU_URI" vol-delete --pool $POOL --vol $NAME
    exit $result
fi

# Change the ownership of the volume to qemu. Without doing this libvirt cannot
# access the volume.
# output=$(chown $USER:$QEMU_GROUP $output 2>1)
# result=$?
# if [[ $result -ne 0 ]]; then
#     echo "Failed to change ownership of the volume to qemu"
#     echo "$output"
#     virsh --connect "$QEMU_URI" vol-delete --pool $POOL --vol $NAME
#     exit $result
# fi

if [[ -n $IMAGE ]]; then
    # Upload an image to the volume.
    output=$(virsh --connect "$QEMU_URI" vol-upload --pool $POOL --vol $NAME --file $IMAGE 2>&1)
    result=$?
    if [[ $result -ne 0 ]]; then
        echo "Failed to upload image $IMAGE to volume $NAME"
        echo "$output"
        virsh --connect "$QEMU_URI" vol-delete --pool $POOL --vol $NAME
        exit $result
    fi

    # Resize the volume to the requested capacity.
    output=$(virsh --connect "$QEMU_URI" vol-resize --pool $POOL --vol $NAME --capacity $CAPACITY 2>&1)
    result=$?
    if [[ $result -ne 0 ]]; then
        echo "Failed to resize volume $VOLUME to $CAPACITY"
        echo "$output"
        virsh --connect "$QEMU_URI" vol-delete --pool $POOL --vol $NAME
        exit $result
    fi
fi

echo '{"changed": true}'
exit 0
